﻿using System.Diagnostics.CodeAnalysis;
using System.Numerics;

class Typ : ICloneable, IComparable<Typ>
//struct Typ
{
    public int A = 1;

    public Typ()
    {
    }

    /*
    //niezgodnie z konwencją dla klas
    //niezbędne dla struktur, jeżeli chcemy je tak porównywać
    public static bool operator ==(Typ left, Typ right)
    {
        return left.A == right.A;
    }

    public static bool operator !=(Typ left, Typ right)
    {
        return !(left == right);
    }
    */

    //ma sens tylko w przypadku klas
    public override bool Equals([NotNullWhen(true)] object? obj)
    {
        if(obj == null) throw new ArgumentNullException(nameof(obj));
        if(obj is Typ) return true;
        return this.A == ((Typ)obj).A;
    }

    public override string ToString()
    {
        return A.ToString();
    }

    public object Clone()
    {
        return new Typ() { A = this.A };
    }

    public int CompareTo(Typ? other)
    {
        if (other == null) throw new ArgumentNullException(nameof(other));
        return this.A.CompareTo(other.A);
    }
}

static class Program
{
    /*
    public static void WyświetlTablicę(this object[] tablica)
    {
        for (int i = 0; i < tablica.Length; ++i)
            Console.WriteLine(tablica[i]?.ToString());
    }
    */

    public static void WyświetlTablicę<T>(this T[] tablica)
    {
        for (int i = 0; i < tablica.Length; ++i)
            Console.WriteLine(tablica[i]?.ToString());
    }

    public static void Main()
    {
        Typ obiekt = new Typ() { A = 1 };
        Typ kopia = (Typ)obiekt.Clone(); //dodatkowa referencja do tego obiektu
        Console.WriteLine(obiekt == kopia);
        Console.WriteLine(obiekt.Equals(kopia));

        /*
        void wyświetlTablicę(Typ[] tablica)
        {
            for(int i = 0; i < tablica.Length; ++i)
                Console.WriteLine(tablica[i].ToString());
        }

        void wyświetlTablicę<T>(T[] tablica)
        {
            //if (tablica == null) throw new ArgumentNullException("Nie lubimy null");
            for (int i = 0; i < tablica.Length; ++i)
                Console.WriteLine(tablica[i]?.ToString());
        }        
        */

        Random r = new Random();
        Typ[] tt = new Typ[10];
        for (int i = 0; i < tt.Length; ++i)
        {
            tt[i] = new Typ() { A = r.Next(10) };
        }
        //wyświetlTablicę(tt);
        tt.WyświetlTablicę();
        Console.WriteLine("Sortowanie");
        Array.Sort(tt);
        //wyświetlTablicę(tt);
        tt.WyświetlTablicę();

        double[] td = new double[10];
        for (int i = 0; i < td.Length; ++i)
        {
            td[i] = 10 * r.NextDouble();
        }
        //wyświetlTablicę(td);        
        td.WyświetlTablicę();
    }
}  